@@ -1,5 +1,5 @@
 use crate::{
-    data::{ConversationInfo, Database, EngineError, DEBUG},
+    data::{ConversationInfo, Database, EngineError, FlowTrigger, DEBUG},
     db_connectors::state::delete_state_key,
     send::send_to_callback_url,
     CsmlBot, CsmlFlow,
@@ -7,10 +7,8 @@ use crate::{
 
 use chrono::{prelude::Utc, SecondsFormat};
 use csml_interpreter::{
-    error_format::{ERROR_SIZE_IDENT, ERROR_NUMBER_AS_KEY, ERROR_KEY_ALPHANUMERIC},
-    data::{ast::Flow, Client, Context,
-        Event, Interval, Memory, Message,
-    },
+    data::{ast::Flow, Client, Context, Event, Interval, Memory, Message},
+    error_format::{ERROR_KEY_ALPHANUMERIC, ERROR_NUMBER_AS_KEY, ERROR_SIZE_IDENT},
     get_step,
     interpreter::json_to_literal,
 };
@@ -27,7 +25,7 @@ use md5::{Digest, Md5};
  * Instead, the memory is saved in bulk at the end of each step or interaction, but we still
  * must allow the user to use the `remembered` data immediately.
  */
-pub fn update_current_context(data: &mut ConversationInfo, memories: &HashMap<String, Memory> ) {
+pub fn update_current_context(data: &mut ConversationInfo, memories: &HashMap<String, Memory>) {
     for (_key, mem) in memories.iter() {
         let lit = json_to_literal(&mem.value, Interval::default(), &data.context.flow).unwrap();
 
@@ -39,23 +37,16 @@ pub fn update_current_context(data: &mut ConversationInfo, memories: &HashMap<St
  * Check if memory key is valid
  */
 pub fn validate_memory_key_format(key: &str) -> Result<(), EngineError> {
-
     if !key.chars().all(|c| c.is_alphanumeric() || c == '_') {
-        return Err(EngineError::Format(
-            ERROR_KEY_ALPHANUMERIC.to_owned(),
-        ))
+        return Err(EngineError::Format(ERROR_KEY_ALPHANUMERIC.to_owned()));
     }
 
     if key.len() > std::u8::MAX as usize {
-        return Err(EngineError::Format(
-            ERROR_SIZE_IDENT.to_owned(),
-        ))
+        return Err(EngineError::Format(ERROR_SIZE_IDENT.to_owned()));
     }
 
     if key.parse::<f64>().is_ok() {
-        return Err(EngineError::Format(
-            ERROR_NUMBER_AS_KEY.to_owned(),
-        ))
+        return Err(EngineError::Format(ERROR_NUMBER_AS_KEY.to_owned()));
     }
 
     Ok(())
@@ -95,12 +86,15 @@ pub fn get_event_content(content_type: &str, metadata: &Value) -> Result<String,
             }
         }
         flow_trigger if flow_trigger == "flow_trigger" => {
-            if let Some(val) = metadata["flow_id"].as_str() {
-                Ok(val.to_string())
-            } else {
-                Err(EngineError::Interpreter(
-                    "no flow_id content in event".to_owned(),
-                ))
+            match serde_json::from_value::<FlowTrigger>(metadata.clone()) {
+                Ok(_flow_trigger) => {
+                    Ok(metadata.to_string())
+                }
+                Err(_) => {
+                    Err(EngineError::Interpreter(
+                        "invalid content for event type flow_trigger: expect flow_id and optional step_id".to_owned(),
+                    ))
+                }
             }
         }
         content_type => Err(EngineError::Interpreter(format!(
@@ -125,10 +119,13 @@ pub fn format_event(json_event: serde_json::Value) -> Result<Event, EngineError>
     let content = json_event["payload"]["content"].to_owned();
 
     let content_value = get_event_content(&content_type, &content)?;
+
     Ok(Event {
         content_type,
         content_value,
         content,
+        ttl_duration: json_event["payload"]["ttl_duration"].as_i64(),
+        low_data_mode: json_event["payload"]["low_data_mode"].as_bool(),
     })
 }
 
@@ -151,10 +148,7 @@ pub fn send_msg_to_callback_url(
         _ => (),
     };
 
-    match serde_json::to_string(&messages) {
-        Ok(string) => send_to_callback_url(data, string.as_bytes()),
-        Err(_err) => (),
-    };
+    send_to_callback_url(data, serde_json::json!(messages))
 }
 
 /**
@@ -197,7 +191,6 @@ pub fn messages_formater(
         "received_at".to_owned(),
         json!(Utc::now().to_rfc3339_opts(SecondsFormat::Millis, true)),
     );
-    map.insert("interaction_id".to_owned(), json!(data.interaction_id));
 
     //tmp
     let mut map_client: Map<String, Value> = Map::new();
@@ -256,11 +249,22 @@ pub fn search_flow<'a>(
     bot: &'a CsmlBot,
     client: &Client,
     db: &mut Database,
-) -> Result<&'a CsmlFlow, EngineError> {
+) -> Result<(&'a CsmlFlow, String), EngineError> {
     match event {
         event if event.content_type == "flow_trigger" => {
             delete_state_key(&client, "hold", "position", db)?;
-            get_flow_by_id(&event.content_value, &bot.flows)
+
+            let flow_trigger: FlowTrigger = serde_json::from_str(&event.content_value)?;
+
+            match get_flow_by_id(&flow_trigger.flow_id, &bot.flows) {
+                Ok(flow) => {
+                    match flow_trigger.step_id {
+                        Some(step_id) => Ok((flow, step_id)),
+                        None => Ok((flow, "start".to_owned())),
+                    }
+                },
+                Err(_) => Ok((get_flow_by_id(&bot.default_flow, &bot.flows)? , "start".to_owned())),
+            }
         }
         event => {
             let mut random_flows = vec![];
@@ -279,7 +283,7 @@ pub fn search_flow<'a>(
             match random_flows.choose(&mut rand::thread_rng()) {
                 Some(flow) => {
                     delete_state_key(&client, "hold", "position", db)?;
-                    Ok(flow)
+                    Ok((flow, "start".to_owned()))
                 }
                 None => Err(EngineError::Interpreter(format!(
                     "Flow '{}' does not exist",
@@ -321,3 +325,48 @@ pub fn clean_hold_and_restart(data: &mut ConversationInfo) -> Result<(), EngineE
     data.context.hold = None;
     return Ok(());
 }
+
+#[cfg(feature = "dynamo")]
+pub fn init_logger() {
+    if let Ok(debug) = env::var("DEBUG") {
+        // RUST_LOG=rusoto
+        // hyper=debug
+        if &debug == "true" {
+            env::set_var("RUST_LOG", "rusoto");
+            env::set_var("hyper", "debug");
+
+            let _ = env_logger::try_init();
+        }
+    };
+}
+
+pub fn get_ttl_duration_value(event: Option<&Event>) -> Option<chrono::Duration> {
+
+    if let Some(event) = event {
+        if let Some(ttl) = event.ttl_duration {
+            return Some(chrono::Duration::days(ttl))
+        }
+    }
+
+    if let Ok(ttl) = env::var("TTL_DURATION") {
+        if let Some(ttl) = ttl.parse::<i64>().ok() {
+            return Some(chrono::Duration::days(ttl))
+        }
+    }
+
+    return None
+}
+
+pub fn get_low_data_mode_value(event: &Event) -> bool {
+    if let Some(low_data) = event.low_data_mode {
+        return low_data;
+    }
+
+    if let Ok(low_data) = env::var("LOW_DATA_MODE") {
+        if let Ok(low_data) = low_data.parse::<bool>() {
+            return low_data;
+        }
+    }
+
+    return false
+}
